import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JPanel;


public class DigitalSignalEncoding extends JPanel {
	
	private static final long serialVersionUID = -6008450455141351888L;

	private final int CLOCK_LINE_WIDTH = 30;
	private final int STREAM_Y = 30;
	private final int STREAM_X = 135;
	private final int POSITIVE_SIGNAL_Y = 50;
	private final int NEGATIVE_SIGNAL_Y = 110;
	private final int ZERO_SIGNAL_Y = 80;
	private final int NRZL = 1;
	private final int NRZI = 2;
	private final int BIPOLAR_AMI = 3;
	private final int PSEUDOTERNARY = 4;
	private final int MANCHESTER = 5;
	private final int DIFFERENTIAL_MANCHESTER = 6;
	private int technique = 0;
	private int STREAM_X2 = 0;
	private char[] stream;
	
	public DigitalSignalEncoding() {
		this.setBackground(Color.white);
	}
	
	// binary data to be represented
	public void setStream(char[] stream) {
		this.stream = stream;
	}
	
	public char[] getStream() {
		return this.stream;
	}
	
	public void setEncodingTechnique(int technique) {
		this.technique = technique;
	}
	
	public int getEncodingTechnique() {
		return this.technique;
	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (this.stream == null || this.technique == -1)
			return;
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setBackground(Color.black);
		drawClockLines(g2);
		switch (this.technique) {
		case NRZL:
			drawPositiveVoltageLevel(g2);
			drawNegativeVoltageLevel(g2);
			nrzlEncoding(g2);
			break;
		case NRZI:
			drawPositiveVoltageLevel(g2);
			drawNegativeVoltageLevel(g2);
			nrziEncoding(g2);
			break;
		case BIPOLAR_AMI:
			drawPositiveVoltageLevel(g2);
			drawNegativeVoltageLevel(g2);
			drawZeroVoltageLevel(g2);
			bipolarAmiEncoding(g2);
			break;
		case PSEUDOTERNARY:
			drawPositiveVoltageLevel(g2);
			drawNegativeVoltageLevel(g2);
			drawZeroVoltageLevel(g2);
			pseudotenaryEncoding(g2);
			break;
		case MANCHESTER:
			drawPositiveVoltageLevel(g2);
			drawNegativeVoltageLevel(g2);
			manchesterEncoding(g2);
			break;
		case DIFFERENTIAL_MANCHESTER:
			drawPositiveVoltageLevel(g2);
			drawNegativeVoltageLevel(g2);
			differentialManchesterEncoding(g2);
			break;
		default:
			break;
		}
	}
	
	public void drawPositiveVoltageLevel(Graphics2D g2) {
		g2.setFont(new Font("Arial", Font.PLAIN, 10));
		g2.setStroke(new BasicStroke(0.5f,
						BasicStroke.CAP_BUTT,
						BasicStroke.JOIN_MITER,
						30.0f,
						new float[] { 8.0f },
						0.0f));
		g2.drawString("+5V", STREAM_X - 30, this.POSITIVE_SIGNAL_Y + 10);
		g2.drawLine(STREAM_X, this.POSITIVE_SIGNAL_Y, STREAM_X2, this.POSITIVE_SIGNAL_Y);
	}
	
	public void drawNegativeVoltageLevel(Graphics2D g2) {
		g2.setFont(new Font("Arial", Font.PLAIN, 10));
		g2.setStroke(new BasicStroke(0.5f,
						BasicStroke.CAP_BUTT,
						BasicStroke.JOIN_MITER,
						30.0f,
						new float[] { 8.0f },
						0.0f));
		g2.drawString("-5V", STREAM_X - 30, this.NEGATIVE_SIGNAL_Y + 5);
		g2.drawLine(STREAM_X, this.NEGATIVE_SIGNAL_Y, STREAM_X2, this.NEGATIVE_SIGNAL_Y);
	}
	
	public void drawZeroVoltageLevel(Graphics2D g2) {
		g2.setFont(new Font("Arial", Font.PLAIN, 10));
		g2.setStroke(new BasicStroke(0.5f,
						BasicStroke.CAP_BUTT,
						BasicStroke.JOIN_MITER,
						30.0f,
						new float[] { 8.0f },
						0.0f));
		g2.drawString("0V", STREAM_X - 25, this.ZERO_SIGNAL_Y + 5);
		g2.drawLine(STREAM_X, this.ZERO_SIGNAL_Y, STREAM_X2, this.ZERO_SIGNAL_Y);
	}
	
	public void nrzlEncoding(Graphics2D g2) {
		int z = this.STREAM_X;
		boolean flag = false;
		g2.setStroke(new BasicStroke(2.0f));
		g2.setFont(new Font("Arial", Font.PLAIN, 15));
		g2.drawString("nrzl", 5, ZERO_SIGNAL_Y);
		for (int i = 0; i < stream.length; i++) {
			if (stream[i] == '1') {
				g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH, this.NEGATIVE_SIGNAL_Y);
				if (!flag)
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);
				flag = true;
			} else {
				g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH, this.POSITIVE_SIGNAL_Y);
				if (flag)
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);
				flag = false;
			}
			z += this.CLOCK_LINE_WIDTH;
		}
	}
	
	public void nrziEncoding(Graphics2D g2) {
		int z = this.STREAM_X;
		int state = this.NEGATIVE_SIGNAL_Y;
		int newState;
		
		g2.setStroke(new BasicStroke(2.0f));
		g2.setFont(new Font("Arial", Font.PLAIN, 15));
		g2.drawString("nrzi", 5, ZERO_SIGNAL_Y);
		
		for (int i = 0; i < stream.length; i++) {
			if (stream[i] == '1') {
				if (state == this.POSITIVE_SIGNAL_Y) {
					newState = this.NEGATIVE_SIGNAL_Y;// change in transition
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);// to show change in transition
					state = newState;
				} else {
					newState = this.POSITIVE_SIGNAL_Y;// change in transition
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);// to show change in transition
					state = newState;
				}
				g2.drawLine(z, newState, z + this.CLOCK_LINE_WIDTH, newState);
			} else {
				// no change in transition
				g2.drawLine(z, state, z + this.CLOCK_LINE_WIDTH, state);
			}
			z += this.CLOCK_LINE_WIDTH;
		}
	}
	
	public void bipolarAmiEncoding(Graphics2D g2) {
		int z = this.STREAM_X;
		int state = this.NEGATIVE_SIGNAL_Y;
		int newState = 0;
		boolean flag = false;
		
		g2.setStroke(new BasicStroke(2.0f));
		g2.setFont(new Font("Arial", Font.PLAIN, 15));
		g2.drawString("bipolarAmi", 5, ZERO_SIGNAL_Y);
		
		for (int i = 0; i < stream.length; i++) {
			if (stream[i] == '1') {
				if (state == this.POSITIVE_SIGNAL_Y) {
					newState = this.NEGATIVE_SIGNAL_Y;// alternative change in transition
					if (!flag)
						g2.drawLine(z, this.ZERO_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);// to show change in transition
					else
						g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);// to show change in
																							// transition
					state = newState;
				} else {
					newState = this.POSITIVE_SIGNAL_Y;// alternative change in transition
					if (!flag)
						g2.drawLine(z, this.ZERO_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);// to show change in transition
					else
						g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);// to show change in
																							// transition
					state = newState;
				}
				g2.drawLine(z, newState, z + this.CLOCK_LINE_WIDTH, newState);
				flag = true;
			} else {
				// Zero level transition
				g2.drawLine(z, this.ZERO_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH, this.ZERO_SIGNAL_Y);
				if (flag)
					g2.drawLine(z, this.ZERO_SIGNAL_Y, z, newState);
				flag = false;
			}
			z += this.CLOCK_LINE_WIDTH;
		}
	}
	
	public void pseudotenaryEncoding(Graphics2D g2) {
		int z = this.STREAM_X;
		int state = this.NEGATIVE_SIGNAL_Y;
		int newState = 0;
		boolean flag = false;
		
		g2.setStroke(new BasicStroke(2.0f));
		g2.setFont(new Font("Arial", Font.PLAIN, 15));
		g2.drawString("pseudotenary", 5, ZERO_SIGNAL_Y);
		
		for (int i = 0; i < stream.length; i++) {
			if (stream[i] == '0') {
				if (state == this.POSITIVE_SIGNAL_Y) {
					newState = this.NEGATIVE_SIGNAL_Y;// alternative change in transition
					if (!flag)
						g2.drawLine(z, this.ZERO_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);// to show change in transition
					else
						g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);// to show change in
																							// transition
					state = newState;
				} else {
					newState = this.POSITIVE_SIGNAL_Y;// alternative change in transition
					if (!flag)
						g2.drawLine(z, this.ZERO_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);// to show change in transition
					else
						g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);// to show change in
																							// transition
					state = newState;
				}
				g2.drawLine(z, newState, z + this.CLOCK_LINE_WIDTH, newState);
				flag = true;
			} else {
				// Zero level transition
				g2.drawLine(z, this.ZERO_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH, this.ZERO_SIGNAL_Y);
				if (flag)
					g2.drawLine(z, this.ZERO_SIGNAL_Y, z, newState);
				flag = false;
			}
			z += this.CLOCK_LINE_WIDTH;
		}
	}
	
	public void manchesterEncoding(Graphics2D g2) {
		int z = this.STREAM_X;
		
		boolean statusHigh = false;
		g2.setStroke(new BasicStroke(2.0f));
		g2.setFont(new Font("Arial", Font.PLAIN, 15));
		g2.drawString("Manchester", 0, ZERO_SIGNAL_Y);
		for (int i = 0; i < stream.length; i++) {
			if (stream[i] == '0') {
				if (statusHigh) {
					/* transition from high to low */
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.POSITIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.NEGATIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					statusHigh = false;
				}// inner if
				else { /* come to the positive level */
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);
					// go from high to low to indicate 0
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.POSITIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.NEGATIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					statusHigh = false;
					
				}// inner else
				
			} else if (stream[i] == '1') {
				if (statusHigh) {
					// come down to the negative level
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);
					// transition from low to high for one
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.NEGATIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.POSITIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					statusHigh = true;
				}// inner if
				else {
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.NEGATIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.POSITIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					statusHigh = true;
					
				}// else with inner if
				
			}// outer if
		}// for loop
	}// function
	
	public void differentialManchesterEncoding(Graphics2D g2) {
		int z = this.STREAM_X;
		
		boolean currentLevelHigh = false;
		g2.setStroke(new BasicStroke(2.0f));
		g2.setFont(new Font("Arial", Font.PLAIN, 15));
		g2.drawString("Diff Manchester", 0, ZERO_SIGNAL_Y);
		for (int i = 0; i < stream.length; i++) {
			if (stream[i] == '0') {
				if (currentLevelHigh) {
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.NEGATIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.POSITIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					
				}// end inner if
				else {
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.POSITIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.NEGATIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					
				}// end else inner
			}// end if
			else {
				if (currentLevelHigh) {
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.POSITIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z, this.NEGATIVE_SIGNAL_Y);
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.NEGATIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					currentLevelHigh = false;
				}// if inner
				else {
					
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.NEGATIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					g2.drawLine(z, this.NEGATIVE_SIGNAL_Y, z, this.POSITIVE_SIGNAL_Y);
					g2.drawLine(z, this.POSITIVE_SIGNAL_Y, z + this.CLOCK_LINE_WIDTH / 2, this.POSITIVE_SIGNAL_Y);
					z += this.CLOCK_LINE_WIDTH / 2;
					currentLevelHigh = true;
				}// else inner
			}// else outer
		}// END OF FOR
	}
	
	public void drawClockLines(Graphics2D g2) {
		g2.setStroke(new BasicStroke(0.5f,
						BasicStroke.CAP_BUTT,
						BasicStroke.JOIN_MITER,
						30.0f,
						new float[] { 8.0f },
						0.0f));
		int x = this.STREAM_X;
		for (int i = 0; i <= stream.length; i++) {
			g2.drawLine(x, 0, x, (int) this.getSize().getHeight());
			x += this.CLOCK_LINE_WIDTH;
		}
		
		this.STREAM_X2 = x - this.CLOCK_LINE_WIDTH;
		if (x > (int) this.getSize().getWidth()) {
			int width = (int) this.getSize().getWidth() + (x - (int) this.getSize().getWidth());
			int height = (int) this.getSize().getHeight();
			this.setPreferredSize(new Dimension(width, height));
			this.revalidate();
		}
		
		x = this.STREAM_X;
		for (int i = 0; i < stream.length; i++) {
			g2.drawString(String.valueOf(stream[i]), x, this.STREAM_Y);
			x += this.CLOCK_LINE_WIDTH + 1;
		}
	}
}